import { IGeodesic } from '../../interfaces/geodesic.interface';
import { Point } from '../../types/point.type';
import { SpacetimePoint } from './spacetime-point';
import { Waypoint } from './waypoint';

export class Route {
  distance: number;
  duration: number;
  fwdAzimuth: number;
  backAzimuth: number;
  distanceAzimuth: number;
  waypoints: Array<Waypoint>;
  points: Array<Point>;
  spacetimePoints: Array<SpacetimePoint>;
  _geodesic: IGeodesic;

  constructor(geodesic: IGeodesic) {
    this.distance = undefined;
    this.duration = undefined;
    this.fwdAzimuth = undefined;
    this.backAzimuth = undefined;
    this.distanceAzimuth = undefined;
    this.waypoints = [];
    this.points = [];
    this.spacetimePoints = [];
    this._geodesic = geodesic;
  }

  setWaypoints = (waypoints: Array<Waypoint>): void => {
    this.waypoints = waypoints;
    this._setAzimuth(waypoints.map((waypoint) => waypoint.point));
  };

  setPoints = (points: Array<Point>): void => {
    this.points = points;
    this._setAzimuth(points);
  };

  setSpacetimePoints = (spacetimePoints: Array<SpacetimePoint>): void => {
    this.spacetimePoints = spacetimePoints;
  };

  _setAzimuth = (points: Array<Point>): void => {
    const inverse = this._geodesic.inverse(
      points[0].lon,
      points[0].lat,
      points[points.length - 1].lon,
      points[points.length - 1].lat,
    );
    this.fwdAzimuth =
      inverse.azimuth >= 0 ? inverse.azimuth : 360 - Math.abs(inverse.azimuth);
    this.backAzimuth =
      this.fwdAzimuth > 180 ? this.fwdAzimuth - 180 : this.fwdAzimuth + 180;
    this.distanceAzimuth = inverse.distance;
  };
}
