import { Injectable } from '@nestjs/common';
import { MatchQuery } from '../../../queries/match.query';
import { Algorithm } from '../../types/algorithm.enum';
import { Match } from '../ecosystem/match';
import { Candidate } from './candidate';
import { AlgorithmFactory } from './factory/algorithm-factory.abstract';
import { ClassicAlgorithmFactory } from './factory/classic';

@Injectable()
export class Matcher {
  match = async (matchQuery: MatchQuery): Promise<Array<Match>> => {
    let algorithm: AlgorithmFactory;
    switch (matchQuery.algorithmSettings.algorithm) {
      case Algorithm.CLASSIC:
        algorithm = new ClassicAlgorithmFactory(matchQuery);
        break;
    }
    let candidates: Array<Candidate> = await algorithm
      .createSelector()
      .select();
    for (const processor of algorithm.createProcessors()) {
      candidates = processor.execute(candidates);
    }
    const match = new Match();
    match.uuid = 'e23f9725-2c19-49a0-9ef6-17d8b9a5ec85';
    return [match];
  };
}
