import { Mapper } from '@automapper/core';
import { InjectMapper } from '@automapper/nestjs';
import { QueryHandler } from '@nestjs/cqrs';
import { Messager } from '../../adapters/secondaries/messager';
import { MatchQuery } from '../../queries/match.query';
import { Match } from '../entities/ecosystem/match';
import { ICollection } from '../../../database/src/interfaces/collection.interface';
import { Matcher } from '../entities/engine/matcher';

@QueryHandler(MatchQuery)
export class MatchUseCase {
  constructor(
    private readonly _matcher: Matcher,
    private readonly _messager: Messager,
    @InjectMapper() private readonly _mapper: Mapper,
  ) {}

  execute = async (matchQuery: MatchQuery): Promise<ICollection<Match>> => {
    try {
      const data: Array<Match> = await this._matcher.match(matchQuery);
      this._messager.publish('matcher.match', 'match !');
      return {
        data,
        total: data.length,
      };
    } catch (error) {
      const err: Error = error;
      this._messager.publish(
        'logging.matcher.match.crit',
        JSON.stringify({
          matchQuery,
          error: err.message,
        }),
      );
      throw error;
    }
  };
}

// const paths = [];
// for (let i = 0; i < 1; i++) {
//   paths.push({
//     key: 'route' + i,
//     points: [
//       {
//         lat: 48.110899,
//         lon: -1.68365,
//       },
//       {
//         lat: 48.131105,
//         lon: -1.690067,
//       },
//       {
//         lat: 48.534769,
//         lon: -1.894032,
//       },
//       {
//         lat: 48.56516,
//         lon: -1.923553,
//       },
//       {
//         lat: 48.622813,
//         lon: -1.997177,
//       },
//       {
//         lat: 48.67846,
//         lon: -1.8554,
//       },
//     ],
//   });
// }
// const routes = await matchQuery.algorithmSettings.georouter.route(paths, {
//   withDistance: false,
//   withPoints: true,
//   withTime: true,
// });
// routes.map((route) => console.log(route.route.spacetimePoints));
